package String::BytewiseNCS;

use strict;
use warnings;
require Exporter;
require DynaLoader;
use AutoLoader;

our @ISA = qw(Exporter DynaLoader);

our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

sub norm_ncs{0.5*(sqrt(8*$_[0]+1)-1)}
# reverce to (n*n+n)/2
#
sub ocs{norm_ncs(ncs(@_))}
our @EXPORT = qw(ncs rawncs);
our $VERSION = '0.99';

bootstrap String::BytewiseNCS $VERSION;

1;
__END__

=head1 NAME

String::BytewiseNCS - An XS implementation of the Levenshtein edit ncs

=head1 SYNOPSIS

 use String::BytewiseNCS qw(ncs);

 print ncs("foo","four");
 # prints "2"


=head1 DESCRIPTION

This module effectively implements the NCS similarity metric 
for short byte strings in a XS way.

The NCS is a maximal possible number of links in a chain of ordered common substrings. 

Look at http://psta.psiras.ru/read/psta2015_1_189-197.pdf
(Znamenskij S. V.: A model and algorithm for
sequence alignment. Program systems: theory and
applications 6:1, 189--197 (2015).)


=head1 CREDITS
 Sergej Znamenskij <svz@latex.pereslavl.ru> 


=head1 SEE ALSO

Znamenskij S. V. Simple essential improvements
to ROUGE-W algorithm Journal of Siberian
Federal University. Mathematics & Physics, 4
(2015) 258--270?

Znamenskij S. V., A Belief Framework for
Similarity Evaluation of Textual or Structured
Data, Similarity Search and Applications LNCS
9371 (2015), 138--149.

Znamenskij S., Dyachenko V. An Alternative Model of the Strings Similarity,
DAMDID/RCDL 2017 (Moscow, Russia, October 9–13, 2017), CEUR Workshop
Proceedings, vol. 2022, Selected Papers of the XIX International Conference on
Data Analytics and Management in Data Intensive Domains, eds. L. Kalinichenko,
Y. Manolopoulos, N. Skvortsov, V. Sukhomlin, pp. 177–183 (in Russian).


=head1 AUTHOR

Copyright 2018 Sergej Znamenskij <F<svz@latex.pereslavl.ru>>

This package is free software and is provided "as is" without express
or implied warranty.  You can redistribute it and/or modify it under 
the same terms as Perl itself.

=cut
