use 5.008;
use strict;
use warnings FATAL => 'all';
use Test::More;
require_ok('String::BytewiseNCS');
subtest 'NCS trap' => sub { 
    is( String::BytewiseNCS::ncs('MONOrkOne','OrkMONOne'), 13, 'NCS trap test');# MONO ne = 13, Ork One =12,
    is( String::BytewiseNCS::ncs('OrkMONOne','MONOrkOne'), 13, 'NCS trap test');# MONO ne = 13, Ork One =12,
    is( String::BytewiseNCS::ncs('xxx' x 1000,'xxa' x 500),      1500,      'test larger source and target');
    is( String::BytewiseNCS::ncs('xxxx','xxxxxx'), 10, 'NCS trap-x test');# MONO ne = 13, Ork One =12,
    is( String::BytewiseNCS::ncs('xxxxxx','xxxx'), 10, 'NCS trap-rx test');# MONO ne = 13, Ork One =12,
};

subtest 'NCS basic examples' => sub { 
    is( String::BytewiseNCS::ncs( 345123456, 123453456  ), 16, 'test NCS with digits');
    is( String::BytewiseNCS::ncs( 123453456,  345123456 ), 16, 'test NCS with digits');
    is( String::BytewiseNCS::ncs('EXTRA TETRAHEDRA','TETRAHEDRAL HEADER'), 55,  'test NCS capitals');
    is( String::BytewiseNCS::ncs('ineffective common efforts','self-finance comes ineffective'),       66,      'test NCS documentation');
    is( String::BytewiseNCS::ncs('mathematical informatics','informatics for mathematics'),            66,      'test NCS documentation');
    is( String::BytewiseNCS::ncs( 12345356,  34512356 ),12,  'test NCS with digits');
    is( String::BytewiseNCS::ncs('',''),                    0,      'test NCS source and target empty');
    is( String::BytewiseNCS::ncs('four',''),                0,      'test NCS four ""');
    is( String::BytewiseNCS::ncs('','four'),                0,      'test NCS "" four');
    is( String::BytewiseNCS::ncs('a','x'),                  0,      'test NCS a x');
    is( String::BytewiseNCS::ncs('aa','x'),                 0,      'test NCS aa x');
    is( String::BytewiseNCS::ncs('a','xx'),                 0,      'test NCS a xx');
    is( String::BytewiseNCS::ncs('aa','xx'),                0,      'test NCS aa xx');
    is( String::BytewiseNCS::ncs('four','four'),           10,      'test NCS matching');
    is( String::BytewiseNCS::ncs('four','fourth'),         10,      'test NCS deletion');
    is( String::BytewiseNCS::ncs('four','fuor'),            3,      'test NCS (no) transposition');
    is( String::BytewiseNCS::ncs('abf','xxsdcd'),           0,      'test NCS abf, xxsdcd');
    is( String::BytewiseNCS::ncs('aasdcd','xbf34872'),      0,      'test NCS aasdcd, xbf34872');
    is( String::BytewiseNCS::ncs('a','x'),                  0,      'test NCS a x');
    is( String::BytewiseNCS::ncs('x','x'),                  1,      'test NCS x x');
    is( String::BytewiseNCS::ncs('a','xa'),                 1,      'test NCS a xa');
    is( String::BytewiseNCS::ncs('ax','x'),                 1,      'test NCS ax x');
    is( String::BytewiseNCS::ncs('xx','x'),                 1,      'test NCS xx x');
    is( String::BytewiseNCS::ncs('x','xx'),                 1,      'test NCS x xx');
    is( String::BytewiseNCS::ncs('xx','xx'),                3,      'test NCS xx xx');
    is( String::BytewiseNCS::ncs(111,11),                   3,      'test NCS numbers');
    is( String::BytewiseNCS::ncs('abcdxx','xx'),            3,      'test NCS abcdxx, xx');
    is( String::BytewiseNCS::ncs('abcdxx','xxz'),           3,      'test NCS abcdxx, xxz');
    is( String::BytewiseNCS::ncs('xx','abcdxx'),            3,      'test NCS xx, abcdxx');
    is( String::BytewiseNCS::ncs('four','for'),             4,      'test NCS insertion');
    is( String::BytewiseNCS::ncs('four','fxxr'),            2,      'test NCS substitution');
    is( String::BytewiseNCS::ncs('four','FOuR'),            1,      'test NCS case');
    is( String::BytewiseNCS::ncs('3684','3668'),            4,      'test46399-94-bad-6-ne-1');
};

subtest 'ncs using utf8' => sub {
    is( String::BytewiseNCS::ncs(' ',' '), 55, 'test NCS with unicode');   
    is( String::BytewiseNCS::ncs(' ',' '), 55, 'test NCS with unicode');
    is( String::BytewiseNCS::ncs('1,2,3,4,5,6,7,8,90','01,2,3,4,5,6,7,8,9'), 153,  'test NCS with short unicode');
};


done_testing();
1;
__END__

 

 MONOrkOne
O * *  *
r    *    
k     *   
M*        
O * *  *
N  *       
O * *  *
n       * 
e        *
  
  
  '), 15, 'ANCS trap test');# MONO ne = 13, Ork One =12,
 1 2 3 4 5 3 4 5 6
3    1     1 
4      3     3  
5        6     6 
11     
2  3     
3    6     6          1
4     10    10        3
5       15    15      6
6               19   10 
                  24=15+15-6
                  
