#!/usr/bin/perl
#
# (c) CC-BY Sergej Znamenskij svz@latex.pereslavl.ru
# 
# File *.txt should be located in direcory "in"
#
use strict;
use warnings;
use utf8;
use Text::Unidecode;
use Data::Dumper;
use String::BytewiseNCS;
$Data::Dumper::Indent =1;
our $interupted=0;
$SIG{INT}=sub{$interupted=1};
my $job;
my $jobtime=time;
#my $debug=1;
sub can_process{
    exit if $interupted;
    return '' if -f "$_[0].bz2" and not -s "$_[0].bz2";
    $job =$_[0];
    unless (-f "$job.bz2"){
        open WJ,">$job.bz2"or die "$! $job.bz2\n";close WJ;}
    open WL, ">>$$.log" or die "$! $$.log\n";
    printf WL    ( -f $_[0] ?  "Reading" : "Creating")." $job started after %d seconds at %s\n", time-$jobtime,  `date +%F.%H-%M-%S`;
    $jobtime=time;
    close WL;
    return 1}

sub result{
    return undef unless defined $job;
    my @r;
    if (-s "$job.bz2"){
        foreach my $s (split "\n",`bzip2 -d -c $job.bz2`){
            chomp $s;
            push @r, [split /\s+/, $s];}
        close I;
        $job=undef;
        return (@r)}
    unless (defined $_[0]){
        $job=undef;
        return undef}
    my $to_sip='';
    if ('ARRAY' eq ref $_[0]){
        $to_sip=(join '', map {join(" ", @$_)."\n"} @_);
        }
    else {$to_sip=join "\n", @_}
    open WJ,"|-", "bzip2 >$job.bz2"or die "$! $job\n";
    print WJ $to_sip ;
    close WJ;
    $job=undef;
    return(@_)}
    

chdir 'in';
foreach my $srcname (<../in/*-*-*.txt>){
    chomp $srcname; 
    next unless $srcname=~/(..\/in\/)(.*?)((\-[a-z][a-z]){1,9}).txt$/;
    my @langs=split /\-/,$3;
    my $filename="$1$2$3";
    chdir '..';     mkdir "$2$3";    chdir "$2$3";
    shift @langs;
    my @data;
    open I, "<:encoding(utf8)", "$filename.txt" or die "$! $filename.txt\n";
    my $tmp='';
    my $ln=0;
    while (<I>){
            chomp;
            $ln++;
            s/\r$// ;
            my @s= map {$_? unidecode($_):'!'} split /\t/, "$_\t.";
            #  my @s=  split /\t/, "$_\t.";
            pop @s;
            if($#s != $#langs){
                print join "'\n\t\t'","Ignore line $ln with $#s+1 entries for $#langs+1 languages;\n'|$_|", @s, "'\n"}
            else{
                $tmp.= join ("\t",@s)."\n";
                push @data, [@s];}}
    close I;
    unless (-s "$filename.ascii"){
        open WJ,">>$filename.ascii"or die "$! $filename.ascii\n";
        print WJ $tmp;
        close WJ;}
    my @summary;
    my %langs;
    my @all_metrics=sort ('NCS','LCS','jw','jwp','lcs','osa','lv','dl','qgram2','qgram3','cosine2','cosine3','qgram1','cosine1');
    my @tables=('1','2','5','10','50','100','200','400', '');
    foreach my $ll(0..$#langs){
        foreach my $metric(@all_metrics){
            foreach my $l(0..$#langs){  
                next if $l==$ll;
                my $fnR ="$langs[$l]-$langs[$ll]-$metric.r";
                my $fn  =   "$metric-$langs[$l]-$langs[$ll].txt";
                next unless &can_process($fn);
                my @dist;
                if ($metric!~/[A-Z]/ and not -s "$fn.bz2"){
                    my $method=($metric=~/([a-z]+)(\d)$/)
                        ? "$1\", q=\"$2"
                        : $metric eq 'jwp'
                            ?   "jw\", p=\"0.1"
                            :   $metric;
                    open W,">$fnR";
                    printf W <<"", $l+1, $l+1, $ll+1, $ll+1;
library(stringdist)
ori<-read.table("$filename.ascii",sep="\\t", quote='',strip.white = TRUE)
m1<-stringdistmatrix(ori[%d]\$V%d,ori[%d]\$V%d, method="$method")
write.table (as.matrix(m1),file="$fn",row.names=FALSE,col.names=FALSE) 
q()

                    close W;
                    `R -f $fnR;sync; bzip2 -f $fn; rm $fnR`}
                elsif(not -s "$fn.bz2"){
                    foreach my $st1 (0..$#data){
                            exit if $interupted;
                        push @dist,[];
                        foreach my $st2 (0..$#data){
                            if($interupted){ die "Interruped!!!\nAborted at $fn.$st1\n"}
                            push @{$dist[$#dist]}, ($metric eq 'LCS')
                                ?  String::BytewiseNCS::lcs($data[$st1][$l],$data[$st2][$ll])
                                :  String::BytewiseNCS::ncs($data[$st1][$l], $data[$st2][$ll]);}}}
                @dist=&result( @dist);
                next unless defined $dist[0];
                foreach my $i (0 .. $#dist){
                    foreach my $j(0 .. $#{$dist[$i]}){ 
                        if ($metric eq 'NCS'){
                             my $m=length($data[$i][$l]);
                             my $d=length($data[$j][$ll]);
                             if ($m>$d){$d=($m-$d); $m-=$d}
                             else{$d-=$m}
                             $dist[$i][$j]= 1.25*$d+$m-$dist[$i][$j];}#   (0.5*(sqrt(8*$dist[$i][$j]) +1)-1)}
                         elsif ($metric eq 'LCS'){
                             my $m=length($data[$i][$l]);
                             my $d=length($data[$j][$ll]);
                             if ($m>$d){$d=($m-$d); $m-=$d}
                             else{$d-=$m}
                             $dist[$i][$j]= $d+$m - $dist[$i][$j]}
                             else{$dist[$i][$j]=~s/NA/1/;}}}
                my @report;
                mkdir "stats";
                next unless &can_process("stats/$langs[$l]-$langs[$ll]-$metric-stat.txt");
                unless( -s "$job.bz2"){
                    my %score;
                    my %totals;
                    foreach my $i(0..$#dist){
                        my $len1=length($data[$i][$ll]);
                        next if $len1<10;
                        foreach my $j(0..$#dist){
                            my $len2=length($data[$j][$ll]);
                            next if $len2<10;
                            my $lratio =exp(int((log (1.0*$len2/$len1))*400 +.5)/100);
                            #printf "$lratio, $len2/$len1, %f\n",(log (1.0*$len2/$len1))*30 +.5;
                            $score{$lratio} =0 unless $score{$lratio};
                            $totals{$lratio}=0 unless $totals{$lratio};
                            $totals{$lratio}+=1;
                            $score{$lratio}-=1 if ($dist[$i][$j]<$dist[$i][$i]);
                            $score{$lratio}+=1 if ($dist[$i][$j]>$dist[$i][$i]);}}
                    foreach my $r (sort {$a <=> $b} keys %score){                    
                        push @report, [ $r, $score{$r}/$totals{$r}, $totals{$r}];}}
                @report=(&result(@report));
                my %summ;
                OUTER: foreach my $report (@report){
                    foreach my $table (reverse @tables){
                        next OUTER if $table and ( $report->[0]<1/(1+0.01*$table) or $report->[0] >1+0.01*$table);
                        $summ{$table}=[0,0] unless $summ{$table};
                        $summ{$table}[0]+=$report->[2]; # total
                        $summ{$table}[1]+=$report->[2]*$report->[1];}}
                $summary[$l][$ll]{$metric}=[map {$summ{$_}} reverse @tables];
            }}
            foreach my $l (0 .. $#langs){
                next if $l==$ll;
                foreach my $metric (@all_metrics){
                    foreach my $tab (0 .. $#tables){
                        $summary[$l][$ll]{average}[$tab][0]=0 unless $summary[$l][$ll]{average}[$tab][0];
                        $summary[$l][$ll]{average}[$tab][1]=0 unless $summary[$l][$ll]{average}[$tab][1];
                        $summary[$l][$ll]{average}[$tab][0]+= ($summary[$l][$ll]{$metric}[$tab][0] ||0)/($#all_metrics+1);
                        $summary[$l][$ll]{average}[$tab][1]+= ($summary[$l][$ll]{$metric}[$tab][1] ||0)/($#all_metrics+1); }}
                $langs{sprintf "%.7f", $summary[$l][$ll]{"average"}[$#tables][1] /($summary[$l][$ll]{average}[$#tables][0] ||1)}=[$l,$ll];}}
        foreach my $t (reverse 0 .. $#tables){
            open O, ">stats/table$tables[$#tables-$t].txt";
            print O join " ", "lang1/lang2", (reverse sort @all_metrics),"average\n";
            foreach my $lkey(sort keys %langs){
                print O "\"$langs[$langs{$lkey}[0]]/$langs[$langs{$lkey}[1]]\"".( join '', 
                    map {sprintf " %.3f", $summary[$langs{$lkey}[0]][$langs{$lkey}[1]]{$_}[$t][1]
                        /($summary[$langs{$lkey}[0]][$langs{$lkey}[1]]{$_}[$t][0] || 1)} 
                        (reverse sort @all_metrics), "average")."\n"}
            close O} 
    chdir "../in"}
__END__

# vim:ts=4:set expandtab

 
